# Copyright 2026 Terradue
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# generated by datamodel-codegen:
#   filename:  oci_annotations.yaml
#   timestamp: 2026-03-10T11:35:32+00:00

from __future__ import annotations

from typing import Optional

from pydantic import Field, RootModel
from transpiler_mate import TranspilerBaseModel


class OciAnnotations(TranspilerBaseModel):
    org_opencontainers_image_title: Optional[str] = Field(
        None, alias="org.opencontainers.image.title"
    )
    org_opencontainers_image_description: Optional[str] = Field(
        None, alias="org.opencontainers.image.description"
    )
    org_opencontainers_image_version: Optional[str] = Field(
        None, alias="org.opencontainers.image.version"
    )
    org_opencontainers_image_source: Optional[str] = Field(
        None, alias="org.opencontainers.image.source"
    )
    org_opencontainers_image_revision: Optional[str] = Field(
        None, alias="org.opencontainers.image.revision"
    )
    org_opencontainers_image_licenses: Optional[str] = Field(
        None, alias="org.opencontainers.image.licenses"
    )
    org_cwl_entrypoint: Optional[str] = Field(None, alias="org.cwl.entrypoint")
    org_cwl_spec: Optional[str] = Field(None, alias="org.cwl.spec")
    org_cwl_type: Optional[str] = Field(None, alias="org.cwl.type")


class Model(RootModel[OciAnnotations]):
    root: OciAnnotations
